@@ -1,9 +1,8 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2005 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdlib.h>
-#include <X11/Xatom.h>
 #include "evilwm.h"
 #include "log.h"
 
@@ -48,10 +47,7 @@ static void handle_key_event(XKeyEvent *e) {
 				- c->height-c->border;
 			goto move_client;
 		case KEY_KILL:
-			if (e->state & ShiftMask)
-				XKillClient(dpy, c->window);
-			else
-				send_wm_delete(c);
+			send_wm_delete(c, e->state & ShiftMask);
 			break;
 		case KEY_LOWER: case KEY_ALTLOWER:
 			XLowerWindow(dpy, c->parent);
@@ -60,14 +56,14 @@ static void handle_key_event(XKeyEvent *e) {
 			show_info(c, key);
 			break;
 		case KEY_MAX:
-			maximise_horiz(c);
+			maximise_client(c, MAXIMISE_HORZ|MAXIMISE_VERT);
+			break;
 		case KEY_MAXVERT:
-			maximise_vert(c);
-			goto move_client;
+			maximise_client(c, MAXIMISE_VERT);
+			break;
 #ifdef VWM
 		case KEY_FIX:
-			c->vdesk = c->vdesk == STICKY ? vdesk : STICKY;
-			client_update_current(c, current);
+			fix_client(c);
 			break;
 #endif
 		}
@@ -97,7 +93,9 @@ static void handle_key_event(XKeyEvent *e) {
 	return;
 move_client:
 	moveresize(c);
-	set_mouse_corner(c);
+	setmouse(c->window, c->width + c->border - 1,
+			c->height + c->border - 1);
+	discard_enter_events();
 	return;
 }
 
@@ -105,7 +103,7 @@ move_client:
 static void handle_button_event(XButtonEvent *e) {
 	Client *c = find_client(e->window);
 
-	if (c && e->window != c->screen->root) {
+	if (c) {
 		switch (e->button) {
 			case Button1:
 				drag(c); break;
@@ -124,6 +122,8 @@ static void handle_configure_request(XConfigureRequestEvent *e) {
 
 	wc.sibling = e->above;
 	wc.stack_mode = e->detail;
+	wc.width = e->width;
+	wc.height = e->height;
 	if (c) {
 		ungravitate(c);
 		if (e->value_mask & CWWidth) c->width = e->width;
@@ -140,17 +140,13 @@ static void handle_configure_request(XConfigureRequestEvent *e) {
 
 		wc.x = c->x - c->border;
 		wc.y = c->y - c->border;
-		wc.width = c->width + (c->border*2);
-		wc.height = c->height + (c->border*2);
 		wc.border_width = 0;
 		XConfigureWindow(dpy, c->parent, e->value_mask, &wc);
 		send_config(c);
 	}
 
-	wc.x = c ? c->border : e->x;
-	wc.y = c ? c->border : e->y;
-	wc.width = e->width;
-	wc.height = e->height;
+	wc.x = c ? 0 : e->x;
+	wc.y = c ? 0 : e->y;
 	XConfigureWindow(dpy, e->window, e->value_mask, &wc);
 	LOG_DEBUG("handle_configure_request() : window configured to %dx%d+%d+%d\n", wc.width, wc.height, wc.x, wc.y);
 }
@@ -175,14 +171,18 @@ static void handle_map_request(XMapRequestEvent *e) {
 static void handle_unmap_event(XUnmapEvent *e) {
 	Client *c = find_client(e->window);
 
+	LOG_DEBUG("handle_unmap_event(): ");
 	if (c) {
 		if (c->ignore_unmap) {
 			c->ignore_unmap--;
+			LOG_DEBUG("ignored (%d ignores remaining)\n", c->ignore_unmap);
 		} else {
-			LOG_DEBUG("handle_unmap_event(): flagging client for removal\n");
+			LOG_DEBUG("flagging client for removal\n");
 			c->remove = 1;
 			need_client_tidy = 1;
 		}
+	} else {
+		LOG_DEBUG("unknown client!\n");
 	}
 }
 
@@ -213,7 +213,7 @@ static void handle_enter_event(XCrossingEvent *e) {
 	current_screen = find_screen(e->root);
 	if ((c = find_client(e->window))) {
 #ifdef VWM
-		if (c->vdesk != vdesk && c->vdesk != STICKY)
+		if (c->vdesk != vdesk && !is_sticky(c))
 			return;
 #endif
 		select_client(c);
