@@ -1,5 +1,5 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2005 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdlib.h>
@@ -42,9 +42,7 @@ int             opt_snap = 0;
 #ifdef SHAPE
 int             have_shape, shape_event;
 #endif
-#ifdef MWM_HINTS
 Atom            mwm_hints;
-#endif
 unsigned int numlockmask = 0;
 static unsigned int grabmask1 = ControlMask|Mod1Mask;
 /* This one is used for per-client mousebutton grabs, so global: */
@@ -89,6 +87,7 @@ int main(int argc, char *argv[]) {
 			new->geometry_mask = 0;
 #ifdef VWM
 			new->vdesk = -1;
+			new->sticky = 0;
 #endif
 			if ((tmp = strchr(argv[i], '/'))) {
 				*(tmp++) = 0;
@@ -115,6 +114,9 @@ int main(int argc, char *argv[]) {
 			i++;
 			if (head_app)
 				head_app->vdesk = atoi(argv[i]);
+		} else if (!strcmp(argv[i], "-s")) {
+			if (head_app)
+				head_app->sticky = 1;
 #endif
 		} else if (!strcmp(argv[i], "-mask1") && i+1<argc) {
 			i++;
@@ -128,14 +130,15 @@ int main(int argc, char *argv[]) {
 			exit(0);
 #endif
 		} else {
-			LOG_INFO("usage: evilwm [-display display] [-term termprog] [-fg foreground]");
+			LOG_INFO("usage: evilwm [-display display] [-term termprog] [-fn fontname]\n");
+			LOG_INFO("              [-fg foreground]");
 #ifdef VWM
 			LOG_INFO(" [-fc fixed]");
 #endif
-			LOG_INFO("\n              [-bg background] [-bw borderwidth] [-snap num]\n");
-			LOG_INFO("              [-mask1 modifiers] [-mask2 modifiers]");
+			LOG_INFO(" [-bg background] [-bw borderwidth]\n");
+			LOG_INFO("              [-snap num] [-mask1 modifiers] [-mask2 modifiers]");
 #ifdef VWM
-			LOG_INFO("\n              [-app name/class] [-g geometry] [-v vdesk]");
+			LOG_INFO("\n              [-app name/class] [-g geometry] [-v vdesk] [-s]");
 #endif
 			LOG_INFO(" [-V]\n");
 			exit((!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))?0:1);
@@ -209,12 +212,14 @@ static void setup_display(void) {
 #ifdef COLOURMAP
 	xa_wm_cmapwins = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);
 #endif
-#ifdef MWM_HINTS
 	mwm_hints = XInternAtom(dpy, _XA_MWM_HINTS, False);
-#endif
 
 	font = XLoadQueryFont(dpy, opt_font);
 	if (!font) font = XLoadQueryFont(dpy, DEF_FONT);
+	if (!font) {
+		LOG_ERROR("couldn't find a font to use: try starting with -fn fontname\n");
+		exit(1);
+	}
 
 	move_curs = XCreateFontCursor(dpy, XC_fleur);
 	resize_curs = XCreateFontCursor(dpy, XC_plus);
@@ -239,14 +244,11 @@ static void setup_display(void) {
 	gv.font = font->fid;
 
 	/* set up root window attributes - same for each screen */
-	attr.event_mask = ChildMask | PropertyChangeMask | EnterWindowMask
 #ifdef COLOURMAP
-		| ColormapChangeMask
+	attr.event_mask = ChildMask | EnterWindowMask | ColormapChangeMask;
+#else
+	attr.event_mask = ChildMask | EnterWindowMask;
 #endif
-#ifdef MOUSE
-		| ButtonMask
-#endif
-		;
 
 	/* SHAPE extension? */
 #ifdef SHAPE
