@@ -1,5 +1,5 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2005 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdlib.h>
@@ -8,9 +8,13 @@
 #include "evilwm.h"
 #include "log.h"
 
-#ifdef MWM_HINTS
+#define MAXIMUM_PROPERTY_LENGTH 4096
+
+static void init_geometry(Client *c);
+static void reparent(Client *c);
+static void *get_property(Window w, Atom property, Atom req_type,
+		unsigned long *nitems_return);
 static PropMwmHints *get_mwm_hints(Window);
-#endif
 #ifdef XDEBUG
 static const char *map_state_string(int map_state);
 static const char *gravity_string(int gravity);
@@ -18,21 +22,17 @@ static void debug_wm_normal_hints(XSizeHints *size);
 #else
 # define debug_wm_normal_hints(s)
 #endif
-static void init_geometry(Client *c);
-static void reparent(Client *c);
 
 void make_new_client(Window w, ScreenInfo *s) {
 	Client *c;
 	char *name;
 	XClassHint *class;
-#ifdef MWM_HINTS
 	PropMwmHints *mhints;
-#endif
 
 	XGrabServer(dpy);
 
 	/* First a bit of interaction with the error handler due to X's
-	 * tendancy to batch event notifications.  We set a global variable to
+	 * tendency to batch event notifications.  We set a global variable to
 	 * the id of the window we're initialising then do simple X call on
 	 * that window.  If an error is raised by this (and nothing else should
 	 * do so as we've grabbed the server), the error handler resets the
@@ -46,12 +46,13 @@ void make_new_client(Window w, ScreenInfo *s) {
 	 * we got a chance to grab the server. */
 	if (initialising == None) {
 		LOG_DEBUG("make_new_client() : XError occurred for initialising window - aborting...\n");
-		XSync(dpy, False);
 		XUngrabServer(dpy);
 		return;
 	}
 	initialising = None;
 	LOG_DEBUG("make_new_client(): %s\n", name ? name : "Untitled");
+	if (name)
+		XFree(name);
 
 	c = malloc(sizeof(Client));
 	/* Don't crash the window manager, just fail the operation. */
@@ -67,35 +68,23 @@ void make_new_client(Window w, ScreenInfo *s) {
 	c->ignore_unmap = 0;
 	c->remove = 0;
 
+	/* Ungrab the X server as soon as possible. Now that the client is
+	 * malloc()ed and attached to the list, it is safe for any subsequent
+	 * X calls to raise an X error and thus flag it for removal. */
+	XUngrabServer(dpy);
+
 	c->border = opt_bw;
-#ifdef MWM_HINTS
 	if ((mhints = get_mwm_hints(c->window))) {
-		if (mhints->flags & MWM_HINTS_DECORATIONS
-				&& !(mhints->decorations & MWM_DECOR_ALL)) {
-			if (!(mhints->decorations & MWM_DECOR_BORDER)) {
-				c->border = 0;
-			}
+		if ((mhints->flags & MWM_HINTS_DECORATIONS)
+				&& !(mhints->decorations & MWM_DECOR_ALL)
+				&& !(mhints->decorations & MWM_DECOR_BORDER)) {
+			c->border = 0;
 		}
 		XFree(mhints);
 	}
-#endif
-	/* If we don't have MWM_HINTS (i.e., lesstif) for a client to tell us
-	 * it has no border, I include this *really blatant hack* to remove
-	 * the border from XMMS. */
-	if (name) {
-#ifndef MWM_HINTS
-		if (!strncmp("XMMS", name, 4))
-			c->border = 0;
-#endif
-		XFree(name);  /* But we want to free this anyway... */
-	}
 
 	init_geometry(c);
 
-#ifdef VWM
-	c->vdesk = vdesk;
-#endif
-
 #ifdef DEBUG
 	{
 		Client *p;
@@ -114,13 +103,6 @@ void make_new_client(Window w, ScreenInfo *s) {
 
 	reparent(c);
 
-	XMapWindow(dpy, c->window);
-	XMapRaised(dpy, c->parent);
-	set_wm_state(c, NormalState);
-
-	XSync(dpy, False);
-	XUngrabServer(dpy);
-
 #ifdef SHAPE
 	if (have_shape) {
 	    XShapeSelectInput(dpy, c->window, ShapeNotifyMask);
@@ -156,9 +138,7 @@ void make_new_client(Window w, ScreenInfo *s) {
 				moveresize(c);
 #ifdef VWM
 				if (a->vdesk != -1) c->vdesk = a->vdesk;
-				if (vdesk != c->vdesk && c->vdesk != STICKY) {
-					hide(c);
-				}
+				c->sticky = a->sticky;
 #endif
 			}
 			a = a->next;
@@ -168,9 +148,24 @@ void make_new_client(Window w, ScreenInfo *s) {
 		XFree(class);
 	}
 
-	/* send_config(c); */
+	/* Only map the window frame (and thus the window) if it's supposed
+	 * to be visible on this virtual desktop. */
+#ifdef VWM
+	if (vdesk == c->vdesk || is_sticky(c))
+#endif
+	{
+		unhide(c, RAISE);
 #ifndef MOUSE
-	set_mouse_corner(c);
+		select_client(c);
+		setmouse(c->window, c->width + c->border - 1,
+				c->height + c->border - 1);
+		discard_enter_events();
+#endif
+	}
+#ifdef VWM
+	else {
+		set_wm_state(c, IconicState);
+	}
 #endif
 }
 
@@ -180,10 +175,13 @@ void make_new_client(Window w, ScreenInfo *s) {
  * XGetWindowAttributes 
  */
 static void init_geometry(Client *c) {
-	CARD32 size_flags;
+	long size_flags;
 	XWindowAttributes attr;
-	XWMHints *wm;
-	int reconfigure = 0;
+
+#ifdef VWM
+	c->vdesk = vdesk;
+	remove_sticky(c);
+#endif
 
 	/* Get current window attributes */
 	LOG_XDEBUG("XGetWindowAttributes()\n");
@@ -204,7 +202,7 @@ static void init_geometry(Client *c) {
 	} else {
 		c->width = c->min_width;
 		c->height = c->min_height;
-		reconfigure = 1;
+		send_config(c);
 	}
 	if ((attr.map_state == IsViewable)
 			|| (size_flags & (/*PPosition |*/ USPosition))) {
@@ -221,22 +219,13 @@ static void init_geometry(Client *c) {
 #else
 		c->x = c->y = c->border;
 #endif
-		reconfigure = 1;
+		send_config(c);
 	}
 
 	LOG_DEBUG("\twindow started as %dx%d +%d+%d\n", c->width, c->height, c->x, c->y);
 	if (attr.map_state == IsViewable) {
+		/* The reparent that is to come would trigger an unmap event */
 		c->ignore_unmap++;
-	} else {
-		/* Initialise position if needed */
-		if (reconfigure) {
-			send_config(c);
-		}
-		if ((wm = XGetWMHints(dpy, c->window))) {
-			if (wm->flags & StateHint)
-				set_wm_state(c, wm->initial_state);
-			XFree(wm);
-		}
 	}
 	gravitate(c);
 }
@@ -244,24 +233,25 @@ static void init_geometry(Client *c) {
 static void reparent(Client *c) {
 	XSetWindowAttributes p_attr;
 
+	p_attr.border_pixel = c->screen->bg.pixel;
 	p_attr.override_redirect = True;
-	p_attr.background_pixel = c->screen->bg.pixel;
 	p_attr.event_mask = ChildMask | ButtonPressMask | ExposureMask | EnterWindowMask;
 	c->parent = XCreateWindow(dpy, c->screen->root, c->x-c->border, c->y-c->border,
-		c->width+(c->border*2), c->height + (c->border*2), 0,
+		c->width, c->height, c->border,
 		DefaultDepth(dpy, c->screen->screen), CopyFromParent,
 		DefaultVisual(dpy, c->screen->screen),
-		CWOverrideRedirect | CWBackPixel | CWEventMask, &p_attr);
+		CWOverrideRedirect | CWBorderPixel | CWEventMask, &p_attr);
 
 	XAddToSaveSet(dpy, c->window);
 	XSetWindowBorderWidth(dpy, c->window, 0);
-	XReparentWindow(dpy, c->window, c->parent, c->border, c->border);
+	XReparentWindow(dpy, c->window, c->parent, 0, 0);
+	XMapWindow(dpy, c->window);
 }
 
 /* Get WM_NORMAL_HINTS property */
-CARD32 get_wm_normal_hints(Client *c) {
+long get_wm_normal_hints(Client *c) {
 	XSizeHints *size;
-	CARD32 flags;
+	long flags;
 	long dummy;
 	size = XAllocSizeHints();
 	LOG_XDEBUG("XGetWMNormalHints()\n");
@@ -305,23 +295,35 @@ CARD32 get_wm_normal_hints(Client *c) {
 	return flags;
 }
 
-#ifdef MWM_HINTS
-static PropMwmHints *get_mwm_hints(Window w) {
+static void *get_property(Window w, Atom property, Atom req_type,
+		unsigned long *nitems_return) {
 	Atom actual_type;
 	int actual_format;
-	unsigned long nitems, bytes_after;
-	PropMwmHints *data;
-	if (XGetWindowProperty(dpy, w, mwm_hints, 0L,
-				(long)PROP_MWM_HINTS_ELEMENTS, False,
-				mwm_hints, &actual_type, &actual_format,
-				&nitems, &bytes_after,
-				(unsigned char **)&data)
-			== Success && nitems >= PROP_MWM_HINTS_ELEMENTS) {
-		return data;
+	unsigned long bytes_after;
+	unsigned char *prop;
+	if (XGetWindowProperty(dpy, w, property,
+				0L, MAXIMUM_PROPERTY_LENGTH / 4, False,
+				req_type, &actual_type, &actual_format,
+				nitems_return, &bytes_after, &prop)
+			== Success) {
+		if (actual_type == req_type)
+			return (void *)prop;
+		XFree(prop);
+	}
+	return NULL;
+}
+
+static PropMwmHints *get_mwm_hints(Window w) {
+	unsigned long nitems;
+	PropMwmHints *prop;
+	if ( (prop = get_property(w, mwm_hints, mwm_hints, &nitems)) ) {
+		if (nitems >= PROP_MWM_HINTS_ELEMENTS) {
+			return prop;
+		}
+		XFree(prop);
 	}
 	return NULL;
 }
-#endif
 
 #ifdef XDEBUG
 static const char *map_state_string(int map_state) {
