@@ -1,5 +1,5 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2005 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdio.h>
@@ -69,6 +69,7 @@ static void remove_info_window(void) {
 }
 #endif  /* INFOBANNER */
 
+#if defined(MOUSE) || !defined(INFOBANNER)
 static void draw_outline(Client *c) {
 #ifndef INFOBANNER_MOVERESIZE
 	char buf[27];
@@ -88,6 +89,7 @@ static void draw_outline(Client *c) {
 		buf, strlen(buf));
 #endif  /* ndef INFOBANNER_MOVERESIZE */
 }
+#endif
 
 #ifdef MOUSE
 void get_mouse_position(int *x, int *y, Window root) {
@@ -206,7 +208,7 @@ static void snap_client(Client *c) {
 	for (ci = head_client; ci; ci = ci->next) {
 		if (ci != c
 #ifdef VWM
-				&& (ci->vdesk == vdesk || ci->vdesk == STICKY)
+				&& (ci->vdesk == vdesk || is_sticky(ci))
 #endif
 				) {
 			if (ci->y - ci->border - c->border - c->height - c->y <= opt_snap && c->y - c->border - ci->border - ci->height - ci->y <= opt_snap) {
@@ -296,87 +298,58 @@ void drag(Client *c) {
 
 void moveresize(Client *c) {
 	XRaiseWindow(dpy, c->parent);
-	XMoveResizeWindow(dpy, c->parent, c->x - c->border,
-			c->y - c->border, c->width + (c->border*2),
-			c->height + (c->border*2));
-	XMoveResizeWindow(dpy, c->window, c->border, c->border,
+	XMoveResizeWindow(dpy, c->parent, c->x - c->border, c->y - c->border,
 			c->width, c->height);
+	XMoveResizeWindow(dpy, c->window, 0, 0, c->width, c->height);
 	send_config(c);
 }
 
-void maximise_horiz(Client *c) {
-	LOG_DEBUG("SCREEN: maximise_horiz()\n");
-	if (c->oldw) {
-		c->x = c->oldx;
-		c->width = c->oldw;
-		c->oldw = 0;
-	} else {
-		c->oldx = c->x;
-		c->oldw = c->width;
-		recalculate_sweep(c, 0, c->y, DisplayWidth(dpy, c->screen->screen),
-				c->y + c->height);
+void maximise_client(Client *c, int hv) {
+	if (hv & MAXIMISE_HORZ) {
+		if (c->oldw) {
+			c->x = c->oldx;
+			c->width = c->oldw;
+			c->oldw = 0;
+		} else {
+			c->oldx = c->x;
+			c->oldw = c->width;
+			c->x = 0;
+			c->width = DisplayWidth(dpy, c->screen->screen);
+		}
 	}
-}
-
-void maximise_vert(Client *c) {
-	LOG_DEBUG("SCREEN: maximise_vert()\n");
-	if (c->oldh) {
-		c->y = c->oldy;
-		c->height = c->oldh;
-		c->oldh = 0;
-	} else {
-		c->oldy = c->y;
-		c->oldh = c->height;
-		recalculate_sweep(c, c->x, 0, c->x + c->width,
-				DisplayHeight(dpy, c->screen->screen));
+	if (hv & MAXIMISE_VERT) {
+		if (c->oldh) {
+			c->y = c->oldy;
+			c->height = c->oldh;
+			c->oldh = 0;
+		} else {
+			c->oldy = c->y;
+			c->oldh = c->height;
+			c->y = 0;
+			c->height = DisplayHeight(dpy, c->screen->screen);
+		}
 	}
+	recalculate_sweep(c, c->x, c->y, c->x + c->width, c->y + c->height);
+	moveresize(c);
+	discard_enter_events();
 }
 
 #ifdef VWM
 void hide(Client *c) {
-	if (c) {
-		c->ignore_unmap++;
-		LOG_XDEBUG("screen:XUnmapWindow(parent); ");
-		XUnmapWindow(dpy, c->parent);
-		set_wm_state(c, IconicState);
-	}
+	/* This will generate an unmap event.  Tell event handler
+	 * to ignore it. */
+	c->ignore_unmap++;
+	LOG_XDEBUG("screen:XUnmapWindow(parent); ");
+	XUnmapWindow(dpy, c->parent);
+	set_wm_state(c, IconicState);
 }
 #endif
 
 void unhide(Client *c, int raise_win) {
-	XMapWindow(dpy, c->window);
 	raise_win ? XMapRaised(dpy, c->parent) : XMapWindow(dpy, c->parent);
 	set_wm_state(c, NormalState);
 }
 
-void set_mouse_corner(Client *c) {
-	int use_x=0, use_y=0;
-#ifdef SHAPE
-	if (c->bounding_shaped) {
-		int i, n_rect, order, maxdiag=0;
-		XRectangle *r = XShapeGetRectangles(dpy, c->window, ShapeBounding,
-				&n_rect, &order);
-		LOG_DEBUG("XShapeGetRectangles: n=%d  r=%p\n", n_rect, r);
-		for (i=0; i<n_rect; i++) {
-			int diag = r[i].x+r[i].width+r[i].y+r[i].width+r[i].height;
-			if (diag > maxdiag) {
-				maxdiag = diag;
-				use_x = r[i].x+r[i].width;
-				use_y = r[i].y+r[i].height;
-			}
-		}
-		XFree(r);
-	} else {
-#endif
-		use_x = c->width  + c->border;
-		use_y = c->height + c->border;
-#ifdef SHAPE
-	}
-#endif
-	LOG_DEBUG("set_mouse_corner: %d %d\n", use_x-1, use_y-1);
-	setmouse(c->window, use_x-1, use_y-1);
-}
-
 void next(void) {
 	Client *newc = current;
 	do {
@@ -393,7 +366,7 @@ void next(void) {
 			return;
 	}
 #ifdef VWM
-	while (newc->vdesk != vdesk && newc->vdesk != STICKY);
+	while (newc->vdesk != vdesk && !is_sticky(newc));
 #else
 	while (0);
 #endif
@@ -402,50 +375,42 @@ void next(void) {
 	unhide(newc, RAISE);
 	select_client(newc);
 	setmouse(newc->window, 0, 0);
-	/* Need to think about this - see note about shaped
-	 * windows in TODO */
-	set_mouse_corner(newc);
+	setmouse(newc->window, newc->width + newc->border - 1,
+			newc->height + newc->border - 1);
+	discard_enter_events();
 }
 
 #ifdef VWM
 void switch_vdesk(int v) {
 	Client *c;
-	int wdesk;
-#ifdef VWM_WARP
-	int warped = 0;
-#endif
 #ifdef DEBUG
 	int hidden = 0, raised = 0;
 #endif
 
 	if (v == vdesk)
 		return;
-	if (current && current->vdesk != STICKY) {
+	if (current && !is_sticky(current)) {
 		client_update_current(current, NULL);
 	}
-	LOG_DEBUG("switch_vdesk() : Switching to desk %d\n", v);
+	LOG_DEBUG("switch_vdesk(): Switching to desk %d", v);
 	for (c = head_client; c; c = c->next) {
-		wdesk = c->vdesk;
-		if (wdesk == vdesk) {
+		if (is_sticky(c) && c->vdesk != v) {
+			c->vdesk = v;
+		}
+		if (c->vdesk == vdesk) {
 			hide(c);
 #ifdef DEBUG
 			hidden++;
 #endif
-		} else if (wdesk == v) {
+		} else if (c->vdesk == v) {
 			unhide(c, NO_RAISE);
 #ifdef DEBUG
 			raised++;
-#endif
-#ifdef VWM_WARP
-			if (!warped) {
-				set_mouse_corner(c);
-				warped = 1;
-			}
 #endif
 		}
 	}
 	vdesk = v;
-	LOG_DEBUG("\tswitch_vdesk() : %d hidden, %d raised\n", hidden, raised);
+	LOG_DEBUG(" (%d hidden, %d raised)\n", hidden, raised);
 }
 #endif /* def VWM */
 
